"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _constants = require("./constants");

const injectEvent = async function (pool, event) {
  const {
    databaseName,
    eventsTableName,
    executeStatement,
    escapeId,
    escape
  } = pool;
  const serializedEvent = [`${escape(event.aggregateId)},`, `${+event.aggregateVersion},`, `${escape(event.type)},`, escape(JSON.stringify(event.payload != null ? event.payload : null))].join('');

  const byteLength = Buffer.byteLength(serializedEvent) + _constants.RESERVED_EVENT_SIZE;

  const databaseNameAsId = escapeId(databaseName);
  const threadsTableAsId = escapeId(`${eventsTableName}-threads`);
  const eventsTableAsId = escapeId(eventsTableName); // prettier-ignore

  const missingFields = [];

  if (event.threadId == null) {
    missingFields.push(`"threadId"`);
  }

  if (event.threadCounter == null) {
    missingFields.push(`"threadCounter"`);
  }

  if (event.timestamp == null) {
    missingFields.push(`"timestamp"`);
  }

  if (missingFields.length > 0) {
    throw new Error(`The field ${missingFields.join(', ')} is required in ${JSON.stringify(event)}`);
  } // prettier-ignore


  await executeStatement(`
    WITH "cte" AS (
      UPDATE ${databaseNameAsId}.${threadsTableAsId} SET
      "threadCounter" = GREATEST("threadCounter", ${+event.threadCounter} + 1)
      WHERE "threadId" = ${+event.threadId}
      RETURNING "threadId"
    )
    INSERT INTO ${databaseNameAsId}.${eventsTableAsId}(
    "threadId",
    "threadCounter",
    "timestamp",
    "aggregateId",
    "aggregateVersion",
    "type",
    "payload",
    "eventSize"
    ) VALUES (
      (SELECT "threadId" FROM "cte" LIMIT 1),
      ${+event.threadCounter},
      ${+event.timestamp},
      ${serializedEvent},
      ${byteLength}
    )
  `);
};

var _default = injectEvent;
exports.default = _default;
//# sourceMappingURL=inject-event.js.map